import numpy as np

import umap
import librosa as lr
import os
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import normalize
import random
import skm

def shingle(X, steps, delay = 1):
    n_points = X.shape[0]
    X = np.pad(X, [(int(steps - 1) * delay, 0), (0, 0)],
        mode="constant", constant_values = [0])
    Xs = X
    for i in range(1, steps):
        Xs = np.hstack([Xs, np.roll(X, -i * delay, axis = 0)])
    return Xs

def unshingle(X, steps, delay = 1):
    n_features = X.shape[1] / steps
    Xu = X[:,:n_features]
    for i in range(1, steps):
        Xtmp = X[:,i*n_features:(i+1)*n_features]
        Xu = np.dstack([Xu, np.roll(Xtmp, i * delay, axis = 0)])
    if steps > 1:
        return np.mean(Xu,2)[steps-1:,:]
    else:
        return Xu

def get_mels(fname):
    y, sr = lr.load(fname.replace("$"," "), sr=44100)
    mels = lr.feature.melspectrogram(y=y, sr=sr, n_mels=n_mels, n_fft=1024)
    return mels

def extract_spectrograms(indexFile):
    global input_matrix
    for line in open(indexFile, "rt"):
        fname = line.split(" ")[0]
        spectrogram = np.log(get_mels(fname).T + np.spacing(1))
        spectrograms.append(spectrogram)
        input_matrix = np.append(input_matrix,shingle(spectrogram, n_shing), axis = 0)

n_mels = 128
n_shing = 2
n_clusters = 128
dsFile = "urban.ds"
spectrograms = []
input_matrix = np.empty((0, n_mels*n_shing))
sk = skm.SKM(k=n_clusters, assignment_change_eps=0.01, normalize=False,
            standardize=True, variance_explained=0.99, visualize=False)

extract_spectrograms("ds/"+dsFile)
pca = PCA(n_components=n_mels*n_shing, whiten=True)

pca.fit(input_matrix)
explained = pca.explained_variance_ratio_.cumsum()
n_features = np.where(explained>0.99)[0][0]
input_matrix = pca.transform(input_matrix)[:,:n_features]
sk.fit(input_matrix.T)
transformed = sk.transform(input_matrix.T).T
transformed = transformed - 0.25
transformed = transformed * (transformed > 0)
output_matrix = np.empty((0, n_clusters * 2))
pos = 0
for s in spectrograms:
    length = s.shape[0]
    tr = transformed[pos:pos + length]
    x = np.concatenate((np.mean(tr,0), np.std(tr,0)))
    output_matrix = np.append(output_matrix, [x], axis = 0)
    pos +=length

np.savetxt("features/"+os.path.splitext(dsFile)[0]+".skmeans", output_matrix)
