# partially adapted from
# https://github.com/isido/dimred/blob/master/dimred/trustcont.py
# Licensed under the GNU General Public License
# (c) 2010 Ilja Sidoroff
# (c) 2021 University of Huddersfield

from scipy.spatial import distance_matrix
import numpy as np
from sklearn.neighbors import KNeighborsClassifier

def moved_out(nn_orig, nn_proj, i, k):
    oo = list(nn_orig[i, 1:k+1])
    pp = list(nn_proj[i, 1:k+1])

    for j in pp:
        if (j in pp) and (j in oo):
            oo.remove(j)

    return oo

def scaling_term(k, n):
    if k < (n / 2.0):
        return 2.0 / ((n*k)*(2*n - 3*k - 1))
    else:
        return 2.0 / (n * (n - k) * (n - k - 1))

def trustcont_sum(moved, ranks, k):
    n = ranks.shape[0]
    s = 0

    # todo: weavefy this for speed
    for i in range(n):
        for j in moved[i]:
            s = s + (ranks[i, j] - k)

    a = scaling_term(k, n)

    return 1 - a * s

def rank_matrix(D):
    mat = np.zeros((D.shape[0], D.shape[1]), dtype='int32')
    for i, row in enumerate(D):
        mat[i] = np.argsort(np.argsort(row))
    return mat


def moved_in(nn_orig, nn_proj, i, k):
    pp = list(nn_proj[i, 1:k+1])
    oo = list(nn_orig[i, 1:k+1])
    for j in oo:
        if (j in oo) and (j in pp):
            pp.remove(j)
    return pp


def trustworthiness(X, Y, ks):
    Xd = distance_matrix(X,X)
    Yd = distance_matrix(Y,Y)
    Xds = np.argsort(Xd)
    Yds = np.argsort(Yd)
    ranks_orig = rank_matrix(Xds)
    trust = []
    for k in ks:
        moved = []
        for i in range(X.shape[0]):
            moved.append(moved_in(Xds, Yds, i, k))

        trust.append(trustcont_sum(moved, ranks_orig, k))

    return trust

def continuity(orig, proj, ks):

    dd_orig = distance_matrix(orig,orig)
    dd_proj = distance_matrix(proj,proj)
    nn_orig = dd_orig.argsort()
    nn_proj = dd_proj.argsort()

    ranks_proj = rank_matrix(dd_proj)

    cont = []
    for k in ks:
        moved = []
        for i in range(orig.shape[0]):
            moved.append(moved_out(nn_orig, nn_proj, i, k))

        cont.append(trustcont_sum(moved, ranks_proj, k))

    return cont

def knn_eval(X, labels, fraction):
    N = X.shape[0]
    M = int(fraction * N)
    perm = np.random.permutation(N)
    train_perm = perm[:M]
    test_perm = perm[M:]
    train_data = X[train_perm,:]
    test_data = X[test_perm,:]
    train_labels = labels[train_perm]
    test_labels = labels[test_perm]
    neigh = KNeighborsClassifier(n_neighbors=2)
    neigh.fit(train_data,train_labels)
    predicted = neigh.predict(test_data)
    acc = np.sum(predicted == test_labels) / len(test_labels)
    return 1-acc
