import sys, os
import numpy as np
from sklearn import  (manifold, decomposition, discriminant_analysis)
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
from numpy import random
import seaborn as sns
import pandas as pd


sns.set_theme()
sns.set_context("paper", font_scale=2.5, rc={"lines.linewidth": 3, "font.weight":"bold"})
sns.set_style("whitegrid")

import umap.umap_ as umap
import eval
from plot import plot_result

classnames = {
#"drums":["clap", "hat", "kick", "ride", "snare"],
"urban":["dog_bark","drilling", "car_horn", "gun_shot"],
"SOL":["Brass", "Keyboards","PluckedStrings", "Strings", "Winds"],
#"gaver":["impact","rolling","scraping","deformation","drip","pour","ripple","splash","woosh","explosion","wind"]
}


for dsname in classnames.keys():
    labels = np.array([line[-2:-1] for line in open("ds/"+dsname+".ds")])
    labelnames = [classnames[dsname][int(i)] for i in labels]
    ncols = len(classnames[dsname])
    cats = pd.Categorical(labelnames)
    um = umap.UMAP(n_components = 2, n_neighbors = 10)
    scaler = StandardScaler()
    Xmfcc = np.loadtxt("features/"+dsname+".mfcc")
    Xmfcc = scaler.fit_transform(Xmfcc)
    Ymfcc = um.fit_transform(Xmfcc)

    Xfl = np.loadtxt("features/"+dsname+".skmeans")
    Xfl = scaler.fit_transform(Xfl)
    Yfl = um.fit_transform(Xfl)

    plt.figure(figsize=(10,5))
    plt.title(dsname.upper(), weight='bold').set_fontsize('24')

    mfcc_result = np.zeros((9, 10))
    for p in range(10):
        r = np.zeros(9)
        for j in range(10):
            r +=  0.1*np.array([eval.knn_eval(Ymfcc, labels, f/10.0) for f in range(1,10)])
        mfcc_result[:,p] = r

    mfcc_result = np.mean(mfcc_result, 1)
    ax = sns.lineplot(np.arange(0.1,1.0, 0.1), mfcc_result, label = "mfcc")
    ax.set(xlabel='Fraction of labeled points', ylabel='Classification error')


    fl_result = np.zeros((9, 10))
    for p in range(10):
        r = np.zeros(9)
        for j in range(10):
            r +=  0.1*np.array([eval.knn_eval(Yfl, labels, f/10.0) for f in range(1,10)])
        fl_result[:,p] = r

    fl_result = np.mean(fl_result, 1)

    sns.lineplot(np.arange(0.1,1.0, 0.1), fl_result, label = "feature learning")
    plt.ylim(0, 0.6)
    sns.despine()
    plt.tight_layout()
    plt.savefig(dsname+"_features.pdf")
