import sys, os
import numpy as np
from sklearn import  (manifold, decomposition, discriminant_analysis)
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from numpy import random
import seaborn as sns
import pandas as pd
from scipy.spatial.distance import cdist
from scipy.optimize import linear_sum_assignment
from matplotlib.ticker import MaxNLocator

import umap
import umap.umap_ as umap
import eval
from plot import plot_result


def make_grid(u, fac = 1):
    xmin = np.min(u[:,0])
    xmax = np.max(u[:,0])
    ymin = np.min(u[:,1])
    ymax = np.max(u[:,1])
    N = u.shape[0]
    N = N * fac
    area = (xmax - xmin) * (ymax - ymin)
    side = np.sqrt(area/N)
    ncols  = (xmax - xmin) / side
    nrows  = (ymax - ymin) / side
    ncols = int(np.floor(ncols))
    nrows = int(np.ceil(nrows))
    ncols = N//nrows
    x = np.arange(N)%ncols
    y = np.arange(N)//ncols
    x = xmin + (x/ncols) * (xmax - xmin)
    y = ymin + (y/nrows) * (ymax - ymin)
    grid = np.hstack((x[:,np.newaxis],y[:,np.newaxis]))
    cost = cdist(u, grid)
    row_ind, col_ind = linear_sum_assignment(cost)
    result = np.vstack((grid[col_ind][:,0],grid[col_ind][:,1])).T
    return result

sns.set_theme()
sns.set_style("whitegrid")
sns.set_context("paper", font_scale=2.5, rc={"lines.linewidth": 3, "font.weight":"bold"})

classnames = {
#"drums":["clap", "hat", "kick", "ride", "snare"],
"urban":["dog_bark","drilling", "car_horn", "gun_shot"],
"SOL":["Brass", "Keyboards","PluckedStrings", "Strings", "Winds"],
#"gaver":["impact","rolling","scraping","deformation","drip","pour","ripple","splash","woosh","explosion","wind"]
}

ext="mfcc"
dsname="urban"

fname = "features/"+dsname+"."+ext
labels = np.array([line[-2:-1] for line in open( "ds/"+dsname+".ds")])
labelnames = [classnames[dsname][int(i)] for i in labels]
ncols = len(classnames[dsname])
cats = pd.Categorical(labelnames)
X = np.loadtxt(fname)
n_points, input_size = X.shape
um = umap.UMAP(n_components = 2, n_neighbors = 10)

plt.figure(figsize=(10,5))
plt.title(dsname.upper(), weight='bold').set_fontsize('24')

Y = um.fit_transform(X)
t = np.array(eval.continuity(X, Y, [5]))
x = []
for i in range(1,5):
    Y1 = make_grid(Y, i)
    t1 = np.array(eval.continuity(X, Y1, [5]))
    x.append(t1[0])

ax = sns.lineplot(range(1,5), x)
ax.axhline(t, linestyle="--", color="black")
ax.set(xlabel='Oversample factor', ylabel='Continuity (k = 5)')
ax.xaxis.set_major_locator(MaxNLocator(integer=True))
plt.ylim(0.8, 1.)
plt.tight_layout()
plt.savefig(dsname+ "_knn_grid.pdf")
