import sys, os
import numpy as np
from sklearn import  (manifold, decomposition, discriminant_analysis)
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from numpy import random
import seaborn as sns
import pandas as pd

import umap
import umap.umap_ as umap
import eval
from plot import plot_result

sns.set_theme()
sns.set_style("whitegrid")
sns.set_context("paper", font_scale=2.5, rc={"lines.linewidth": 3, "font.weight":"bold"})

classnames = {
#"drums":["clap", "hat", "kick", "ride", "snare"],
"urban":["dog_bark","drilling", "car_horn", "gun_shot"],
"SOL":["Brass", "Keyboards","PluckedStrings", "Strings", "Winds"],
#"gaver":["impact","rolling","scraping","deformation","drip","pour","ripple","splash","woosh","explosion","wind"]
}

ext="mfcc"
for dsname in classnames.keys():
    fname = "features/"+dsname+"."+ext
    labels = np.array([line[-2:-1] for line in open( "ds/"+dsname+".ds")])
    labelnames = [classnames[dsname][int(i)] for i in labels]
    ncols = len(classnames[dsname])
    cats = pd.Categorical(labelnames)

    X = np.loadtxt(fname)
    n_points, input_size = X.shape

    pca = PCA(n_components=2)
    tsne = manifold.TSNE(n_components = 2, perplexity = 100)
    mds = manifold.MDS(n_components = 2)
    iso = manifold.Isomap(n_components = 2)
    um = umap.UMAP(n_components = 2, n_neighbors = 10)

    algorithms = [pca, mds, iso, tsne, um]
    algonames = ["PCA", "MDS", "Isomap", "t-SNE", "UMAP"]

    plt.figure(figsize=(10,5))
    plt.title(dsname.upper(), weight='bold').set_fontsize('24')


    for i, alg in enumerate(algorithms):
        Y = alg.fit_transform(X)
        t = np.array(eval.trustworthiness(X, Y, range(1, 10)))
        ax = sns.lineplot(range(1,10), t, label = algonames[i])
        ax.set(xlabel='Number of neighbors (k)', ylabel='Trustworthiness')
        plt.ylim(0.5, 1)
    plt.tight_layout()
    plt.savefig(dsname + "_TW.pdf")
