import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import seaborn as sns
import umap
import umap.umap_ as umap
from scipy.optimize import linear_sum_assignment
from scipy.spatial.distance import cdist

fig, axes = plt.subplots(2, 3, figsize = (15, 10), dpi=300)

np.random.seed(42)
data = np.random.rand(500, 4)


fit = umap.UMAP()
u = fit.fit_transform(data)


def make_grid(u, fac = 1):
    xmin = np.min(u[:,0])
    xmax = np.max(u[:,0])
    ymin = np.min(u[:,1])
    ymax = np.max(u[:,1])
    N = u.shape[0]
    N = N * fac
    area = (xmax - xmin) * (ymax - ymin)
    side = np.sqrt(area/N)
    ncols  = (xmax - xmin) / side
    nrows  = (ymax - ymin) / side
    ncols = int(np.floor(ncols))
    nrows = int(np.ceil(nrows))
    ncols = N//nrows
    x = np.arange(N)%ncols
    y = np.arange(N)//ncols
    x = xmin + (x/ncols) * (xmax - xmin)
    y = ymin + (y/nrows) * (ymax - ymin)
    grid = np.hstack((x[:,np.newaxis],y[:,np.newaxis]))
    cost = cdist(u, grid)
    row_ind, col_ind = linear_sum_assignment(cost)
    result = np.vstack((grid[col_ind][:,0],grid[col_ind][:,1])).T
    return result

axi = plt.subplot(2, 3, 6)
plt.title("original").set_fontsize('24')
g = plt.scatter( u[:,0],u[:,1], c=data, marker=".", s=50)
ax = plt.gca()
ax.axes.xaxis.set_visible(False)
ax.axes.yaxis.set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.spines['bottom'].set_visible(False)
ax.spines['left'].set_visible(False)

for i in range(1,6):
    axi = plt.subplot(2, 3, i)
    X = make_grid(u, i)
    plt.title(str(i)+"x ").set_fontsize('24')
    g = plt.scatter( X[:,0],X[:,1], c=data, marker=".", s=50)
    ax = plt.gca()
    ax.axes.xaxis.set_visible(False)
    ax.axes.yaxis.set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.spines['left'].set_visible(False)
plt.tight_layout()
plt.savefig("grid_demo.pdf")
